
import { ExtractJwt, Strategy } from 'passport-jwt';
import { PassportStrategy } from '@nestjs/passport';
import { Injectable, UnauthorizedException } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { UsersService } from 'src/users/users.service';

@Injectable()
export class JwtAccessStrategy extends PassportStrategy(Strategy, 'jwt-access') {
  constructor(private users: UsersService, private config: ConfigService) {
    super({
      jwtFromRequest: ExtractJwt.fromExtractors([
        //ExtractJwt.fromAuthHeaderAsBearerToken(),
        JwtAccessStrategy.extract,
      ]),
      ignoreExpiration: false,
      secretOrKey: config.getOrThrow<string>('AUTH_JWT_ACCESS_TOKEN_SECRET'),
      issuer: config.getOrThrow<string>('AUTH_JWT_ISSUER'),
      audience: config.getOrThrow<string>('AUTH_JWT_AUDIENCE'),
    });
  }

  private static extract(req: any): string | null {
    const jwt = req.cookies?.Authentication;
    if (!jwt)
      return null;

    return jwt;
  }

  async validate(payload: any) {
    if (!payload) {
      throw new UnauthorizedException();
    }

    if (payload.iss != this.config.getOrThrow('AUTH_JWT_ISSUER')) {
      throw new UnauthorizedException();
    }

    if (payload.aud != this.config.getOrThrow('AUTH_JWT_AUDIENCE')) {
      throw new UnauthorizedException();
    }

    const user = await this.users.findById(payload.sub);
    if (!user || user.userLogin != payload.username) {
      throw new UnauthorizedException();
    }
    return user;
  }
}
