import { UUID } from 'crypto';
import { Column, Entity, JoinColumn, ManyToOne, OneToMany, OneToOne, PrimaryColumn, Unique } from 'typeorm';
import { BookOriginEntity } from './book-origin.entity';
import { BookStatusEntity } from './book-status.entity';
import { SeriesEntity } from 'src/series/entities/series.entity';

@Entity("books")
@Unique(['providerSeriesId', 'providerBookId'])
export class BookEntity {
  @PrimaryColumn({ name: 'book_id', type: 'uuid' })
  readonly bookId: UUID;

  @Column({ name: 'provider_series_id', type: 'text', nullable: true })
  providerSeriesId: string;

  @Column({ name: 'provider_book_id', type: 'text', nullable: false })
  providerBookId: string;

  @Column({ name: 'book_title', type: 'text', nullable: false })
  title: string;

  @Column({ name: 'book_desc', type: 'text', nullable: true })
  desc: string;

  @Column({ name: 'book_volume', type: 'integer', nullable: true })
  volume: number;

  @Column({ name: 'provider', type: 'varchar', nullable: false })
  provider: string;

  @Column({ name: 'published_at', type: 'timestamptz', nullable: false })
  publishedAt: Date

  @Column({ name: 'added_at', type: 'timestamptz', nullable: false })
  addedAt: Date;

  @OneToMany(type => BookOriginEntity, origin => origin.book)
  metadata: BookOriginEntity[];

  @OneToMany(type => BookStatusEntity, status => status.book)
  statuses: BookStatusEntity[];

  @OneToOne(type => SeriesEntity, series => series.volumes)
  @JoinColumn([{
    name: 'provider_series_id',
    referencedColumnName: 'providerSeriesId',
  },
  {
    name: 'provider',
    referencedColumnName: 'provider',
  }])
  series: SeriesEntity;
}