import { Module } from '@nestjs/common';
import { LibraryService } from './library.service';
import { HttpModule } from '@nestjs/axios';
import { TypeOrmModule } from '@nestjs/typeorm';
import { BookOriginEntity } from 'src/books/entities/book-origin.entity';
import { BookStatusEntity } from 'src/books/entities/book-status.entity';
import { BookEntity } from 'src/books/entities/book.entity';
import { ProvidersModule } from 'src/providers/providers.module';
import { SeriesModule } from 'src/series/series.module';
import { SeriesEntity } from 'src/series/entities/series.entity';
import { SeriesSubscriptionEntity } from 'src/series/entities/series-subscription.entity';
import { BooksService } from 'src/books/books.service';
import { SeriesService } from 'src/series/series.service';
import { BullModule } from '@nestjs/bullmq';
import { LibraryConsumer } from './library.consumer';
import { LibraryController } from './library.controller';

@Module({
  imports: [
    TypeOrmModule.forFeature([
      BookEntity,
      BookOriginEntity,
      BookStatusEntity,
      SeriesEntity,
      SeriesSubscriptionEntity,
    ]),
    BullModule.registerQueue({
      name: 'library',
    }),
    SeriesModule,
    HttpModule,
    ProvidersModule,
  ],
  exports: [LibraryService],
  providers: [LibraryService, BooksService, SeriesService, LibraryConsumer],
  controllers: [LibraryController]
})
export class LibraryModule { }
