import { UserEntity } from "./users/entities/users.entity";

export function serialize_user_short(value: UserEntity) {
  if (!value) {
    return value;
  }

  return value.userLogin;
}

export function serialize_user_long(value: UserEntity) {
  if (!value) {
    return value;
  }

  return {
    user_id: value.userId,
    user_login: value.userLogin,
    is_admin: value.isAdmin,
  }
}

export function serialize_token(value: string) {
  if (!value) return null;
  return '...' + value.substring(Math.max(value.length - 12, value.length / 2) | 0);
}

export function serialize_req(value) {
  if (!value) {
    return value;
  }

  value = { ...value };

  delete value['remoteAddress']
  delete value['remotePort']

  if (value.headers) {
    const headers = value.headers = { ...value.headers };
    if (headers['authorization']) {
      headers['authorization'] = '...' + headers['authorization'].substring(Math.max(0, headers['authorization'].length - 16))
    }

    if (headers['cookie']) {
      const cookies = headers['cookie'].split(';')
        .map((c: string) => {
          c = c.trim();
          const index = c.indexOf('=');
          if (index < 0)
            return c;

          const cookieValue = c.substring(index + 1);
          return c.substring(0, index) + '=...' + cookieValue.substring(Math.max(cookieValue.length - 16, cookieValue.length / 2));
        });
      headers['cookie'] = cookies.join('; ');
    }
  }
  return value;
}

export function serialize_res(value) {
  if (!value || !value.headers) {
    return value;
  }

  value = { ...value };
  const headers = value.headers = { ...value.headers };
  delete headers['x-powered-by'];

  if (headers['set-cookie']) {
    const cookies = headers['set-cookie'];
    for (let i in cookies) {
      const cookie: string = cookies[i];
      if (cookie.startsWith('Authentication=')) {
        cookies[i] = 'Authentication=...' + cookie.substring(Math.max(0, cookie.indexOf(';') - 12));
      } else if (cookie.startsWith('Refresh=')) {
        cookies[i] = 'Refresh=...' + cookie.substring(Math.max(0, cookie.indexOf(';') - 12));
      }
    }
  }
  return value;
}

export function serialize_job(value) {
  if (!value) {
    return value;
  }

  return {
    id: value.id,
    name: value.name,
    data: value.data,
  }
}
