import * as argon2 from 'argon2';
import * as crypto from 'crypto';
import { UUID } from "crypto";
import { BookStatusEntity } from 'src/books/entities/book-status.entity';
import { BigIntTransformer } from 'src/shared/transformers/bigint';
import { StringToLowerCaseTransformer } from 'src/shared/transformers/string';
import { BeforeInsert, Column, Entity, OneToMany, PrimaryGeneratedColumn } from "typeorm";

@Entity({
  name: 'users'
})
export class UserEntity {
  @PrimaryGeneratedColumn("uuid", { name: 'user_id' })
  userId: UUID;

  @Column({ name: 'user_login', unique: true, transformer: StringToLowerCaseTransformer })
  userLogin: string;

  @Column({ name: 'user_name', nullable: false })
  userName: string;

  @Column({ name: 'password', nullable: false })
  password: string;

  @Column({ name: 'salt', type: 'bigint', nullable: false, transformer: BigIntTransformer })
  salt: BigInt;

  @Column({ name: 'is_admin', nullable: false })
  isAdmin: boolean;

  @Column({ name: 'joined_at', type: 'timestamptz', nullable: false })
  joinedAt: Date;

  @OneToMany(type => BookStatusEntity, bookStatus => bookStatus.userId)
  bookStatuses: BookStatusEntity[];

  @BeforeInsert()
  async hashPassword() {
    const saltBuf = crypto.randomBytes(8);
    // Remove first bit to only have positive, signed 63-bit values.
    saltBuf[0] |= 0b10000000;
    saltBuf[0] ^= 0b10000000;
    this.salt = BigInt("0x" + saltBuf.toString('hex'));

    const buffer = Buffer.concat([
      Buffer.from(this.password, 'utf8'),
      Buffer.from(this.salt.toString(16), 'hex'),
    ]);

    this.password = await argon2.hash(buffer, { raw: false });
  }
}
