import * as argon2 from 'argon2';
import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { UserEntity } from './entities/users.entity';
import { UUID } from 'crypto';
import { LoginDto } from 'src/auth/dto/login.dto';

class UserDto {
  userId: UUID;
  userLogin: string;
  userName: string;
  isAdmin: boolean;
  joinedAt: Date;
}

@Injectable()
export class UsersService {
  constructor(
    @InjectRepository(UserEntity)
    private readonly userRepository: Repository<UserEntity>,
  ) { }

  async findAll(): Promise<UserDto[]> {
    const result = await this.userRepository.find();
    return result.map((u: UserEntity) => ({
      userId: u.userId,
      userLogin: u.userLogin,
      userName: u.userName,
      isAdmin: u.isAdmin,
      joinedAt: u.joinedAt,
    }));
  }

  async findOne(loginDetails: LoginDto): Promise<UserEntity> {
    const user = await this.userRepository.findOneBy({ userLogin: loginDetails.user_login });
    if (!user) {
      // TODO: force an argon2.verify() to occur here.
      return null;
    }

    const buffer = Buffer.concat([
      Buffer.from(loginDetails.password, 'utf8'),
      Buffer.from(user.salt.toString(16), 'hex'),
    ]);
    
    if (await argon2.verify(user.password, buffer)) {
      return user;
    }

    return null;
  }

  async findById(userId: UUID): Promise<UserEntity> {
    const user = await this.userRepository.findOneBy({ userId });
    if (!user) {
      return null;
    }
    return user;
  }

  async register(userLogin: string, userName: string, password: string, isAdmin: boolean) {
    const result = await this.userRepository.create({
      userLogin,
      userName,
      password,
      salt: 0,
      isAdmin,
    });
    return await this.userRepository.save(result);
  }
}
