import { HttpClient } from '@angular/common/http';
import { Component, inject, OnInit } from '@angular/core';
import { FormControl, FormGroup, ReactiveFormsModule, Validators } from '@angular/forms';
import { NoticeComponent } from "../../shared/notice/notice.component";
import { Router } from '@angular/router';
import { ConfigService } from '../../services/config.service';
import { AuthService } from '../../services/auth/auth.service';
import { LoadingService } from '../../services/loading.service';
import { RedirectionService } from '../../services/redirection.service';

@Component({
  selector: 'register-form',
  standalone: true,
  imports: [
    NoticeComponent,
    ReactiveFormsModule,
  ],
  templateUrl: './register-form.component.html',
  styleUrl: './register-form.component.css'
})
export class RegisterFormComponent implements OnInit {
  private readonly _auth = inject(AuthService);
  private readonly _loading = inject(LoadingService);
  private readonly _http = inject(HttpClient);
  private readonly _redirect = inject(RedirectionService);
  private readonly _router = inject(Router);
  readonly config = inject(ConfigService).config;

  readonly name = new FormControl('', [Validators.required]);
  readonly username = new FormControl('', [Validators.required]);
  readonly password = new FormControl('', [Validators.required]);
  readonly passwordConfirmation = new FormControl('', [Validators.required]);
  readonly forms = new FormGroup({
    name: this.name,
    username: this.username,
    password: this.password,
    passwordConfirmation: this.passwordConfirmation
  });

  waitForResponse: boolean = false;
  errorMessage: string = '';

  ngOnInit(): void {
  }

  register() {
    if (this.waitForResponse) {
      return;
    }

    this.waitForResponse = true;
    this.errorMessage = '';
    if (!this.forms.valid) {
      this.waitForResponse = false;
      return;
    }

    if (this.password.value != this.passwordConfirmation.value) {
      this.errorMessage = 'Password confirmation does not match original password.';
      return;
    }

    this._http.post('/api/auth/register', {
      user_login: this.name.value,
      user_name: this.username.value,
      password: this.password.value,
    }).subscribe({
      next: async (response: any) => {
        this._auth.update(true)
          .subscribe(async () => await this._redirect.redirect(null));
      },
      error: (err) => this.handleRegistrationErrors(err),
      complete: () => this.waitForResponse = false
    });
  }

  handleRegistrationErrors(err: any) {
    if (err.status == 401) {
      this.errorMessage = 'Invalid credentials.';
    } else if (err.status == 400 || err.status == 403) {
      this.errorMessage = 'Something went wrong.';
    } else {
      this.errorMessage = 'Something went wrong: ' + err.error.statusCode + ' ' + err.error.message;
    }
    this.waitForResponse = false;
  }

  async login(event: any) {
    event.preventDefault();
    await this._router.navigate(['login']);
  }
}
