import { Injectable } from '@angular/core';
import { BehaviorSubject, delay, filter, first } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class LoadingService {
  private readonly _subject: BehaviorSubject<boolean> = new BehaviorSubject<boolean>(false);
  private readonly _status = this._subject.asObservable().pipe(delay(0));
  private readonly _set: Set<string> = new Set<string>();


  listen() {
    return this._status;
  }

  listenUntilReady() {
    return this._status.pipe(
      filter(status => !status),
      first(),
    );
  }

  /**
   * Updates the subject based on the following:
   * - If loading is true, add url to the set & update subject value to true.
   * - If loading is false, remove url entry and update subject to false if set is empty.
   * This supports multiple requests awaiting completion.
   * 
   * @param loading {boolean}
   * @param url {string}
   */
  set(loading: boolean, url: string): void {
    if (!url) {
      throw new Error('URL must be provided.');
    }
    if (loading) {
      this._set.add(url);
      this._subject.next(true);
    } else if (this._set.has(url)) {
      this._set.delete(url);
      if (this._set.size === 0) {
        this._subject.next(false);
      }
    }
  }
}
