import { inject, Injectable } from '@angular/core';
import { AuthService } from './auth/auth.service';
import { Router } from '@angular/router';

@Injectable({
  providedIn: 'root'
})
export class RedirectionService {
  private readonly _auth = inject(AuthService);
  private readonly _router = inject(Router);

  constructor() { }

  async redirect(destination: string | string[] | null) {
    let uri = this._router.url;
    if (!this._router.url.startsWith('/') && !this._router.url.startsWith('http')) {
      uri = '/' + this._router.url;
    }

    if (await this.handleSpecialCases(uri)) {
      return;
    }

    const queryParams = this._router.parseUrl(uri).queryParams;
    const redirect = queryParams['rd'];
    if (redirect) {
      if (!(await this.handleSpecialCases(redirect))) {
        await this._router.navigateByUrl(redirect);
      }
    } else if (typeof destination === 'string') {
      await this._router.navigateByUrl(destination);
    } else if (destination !== null) {
      await this._router.navigate(destination);
    }
  }

  private async handleSpecialCases(uri: string): Promise<boolean> {
    if (this._auth.isLoggedIn()) {
      if (uri.startsWith('/login') || uri.startsWith('/register')) {
        await this._router.navigate(['add', 'new']);
        return true;
      }
    } else if (!uri.startsWith('/login') && !uri.startsWith('/register')) {
      await this._router.navigate(['login'],
        {
          queryParams: {
            rd: uri,
          }
        });
      return true;
    }

    return false;
  }
}
